/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/***************************************************************************
 *            img_encoder.cc
 *
 *  Mon Nov 15 19:45:07 CET 2004
 *  Copyright  2004 Bent Bisballe
 *  deva@aasimon.org
 ****************************************************************************/

/*
 * Originally from:
 * RTVideoRec Realtime video recoder and encoder for Linux
 *
 * Copyright (C) 2004  B. Stultiens
 * Copyright (C) 2004  Koen Otter and Glenn van der Meyden
 */

/*
 *    This file is part of MIaV.
 *
 *    MIaV is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MIaV is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MIaV; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 */

/*
 * $Id: img_encoder.cc,v 1.10 2005/06/19 20:04:43 deva Exp $
 */

/*
 * $Log: img_encoder.cc,v $
 * Revision 1.10  2005/06/19 20:04:43  deva
 * ImgEncoder now uses the file class for output, through jpeg_mem_dest.
 *
 * Revision 1.9  2005/05/26 12:48:36  deva
 * *** empty log message ***
 *
 * Revision 1.8  2005/05/07 10:25:34  deva
 *
 * Removed ffmpeg code from img_encoder and corrected decoding errors in mov_encoder
 *
 * Revision 1.7  2005/05/03 08:31:59  deva
 * Removed the error object, and replaced it with a more generic info object.
 *
 * Revision 1.6  2005/05/01 09:56:26  deva
 * Added Id and Log tags to all files
 */

#include "img_encoder.h"
#include <stdio.h>

#include "debug.h"

// Use libdv
#include <libdv/dv.h>
#include <libdv/dv_types.h>

ImgEncoder::ImgEncoder(const char* cpr, Info *i)
{
  info = i;

  // Create path and filename
  char fname[256];
  string *server_root;
  char birthmonth[3];
  char date[32];

  // Get server root
  server_root = config->readString("server_root");

  // Copy the bytes representing the birth month from the cpr
  // [dd][mm][yy]-[nn][nn]
  strncpy(birthmonth, &cpr[2], 2);
  birthmonth[2] = 0;

  // Create date (today) in [yyyy][mm][dd]
  struct tm *ltime;
  time_t t = time(NULL);
  ltime = localtime(&t);
  sprintf(date, "%.4d%.2d%.2d", 
          ltime->tm_year + 1900, 
          ltime->tm_mon, 
          ltime->tm_mday);

  sprintf(fname, "%s/%s/%s/%s-%s-", server_root->c_str(), birthmonth, cpr, cpr, date);

  file = new File(fname, "jpg", info);
}


ImgEncoder::~ImgEncoder()
{
  delete file;
}


void ImgEncoder::encode(Frame *dvframe, int quality)
{ 
  unsigned char rgb[720*576*4];

  getRGB(dvframe, rgb);
  writeJPEGFile(quality, (JSAMPLE*)rgb, 720, 576);
}

#define OUTPUT_BUF_SIZE  4096	/* choose an efficiently ?? size */

/* Expanded data destination object for stdio output */
typedef struct {
  struct jpeg_destination_mgr pub; /* public fields */

  JOCTET * outbuff;		/* target buffer */
  size_t * size;
} mem_destination_mgr;

typedef mem_destination_mgr * mem_dest_ptr;

/*
 * Initialize destination --- called by jpeg_start_compress
 * before any data is actually written.
 */
void init_destination (j_compress_ptr cinfo)
{
  mem_dest_ptr dest = (mem_dest_ptr) cinfo->dest;

  *dest->size = 0;
  dest->pub.next_output_byte = dest->outbuff;
  dest->pub.free_in_buffer = OUTPUT_BUF_SIZE;
}

/*
 * Terminate destination --- called by jpeg_finish_compress
 * after all data has been written.  Usually needs to flush buffer.
 *
 * NB: *not* called by jpeg_abort or jpeg_destroy; surrounding
 * application must deal with any cleanup that should happen even
 * for error exit.
 */
void term_destination (j_compress_ptr cinfo)
{
  mem_dest_ptr dest = (mem_dest_ptr) cinfo->dest;
  size_t datacount = OUTPUT_BUF_SIZE - dest->pub.free_in_buffer;

  /* Write any data remaining in the buffer */
  if (datacount > 0) {
    dest->outbuff+=datacount;
    *dest->size+=datacount;
  }
}

/*
 * Empty the output buffer --- called whenever buffer fills up.
 *
 * In typical applications, this should write the entire output buffer
 * (ignoring the current state of next_output_byte & free_in_buffer),
 * reset the pointer & count to the start of the buffer, and return TRUE
 * indicating that the buffer has been dumped.
 *
 * In applications that need to be able to suspend compression due to output
 * overrun, a FALSE return indicates that the buffer cannot be emptied now.
 * In this situation, the compressor will return to its caller (possibly with
 * an indication that it has not accepted all the supplied scanlines).  The
 * application should resume compression after it has made more room in the
 * output buffer.  Note that there are substantial restrictions on the use of
 * suspension --- see the documentation.
 *
 * When suspending, the compressor will back up to a convenient restart point
 * (typically the start of the current MCU). next_output_byte & free_in_buffer
 * indicate where the restart point will be if the current call returns FALSE.
 * Data beyond this point will be regenerated after resumption, so do not
 * write it out when emptying the buffer externally.
 */
boolean empty_output_buffer (j_compress_ptr cinfo)
{
  mem_dest_ptr dest = (mem_dest_ptr) cinfo->dest;

  dest->outbuff+=OUTPUT_BUF_SIZE;
  *dest->size+=OUTPUT_BUF_SIZE;

  dest->pub.next_output_byte = dest->outbuff;
  dest->pub.free_in_buffer = OUTPUT_BUF_SIZE;

  return TRUE;
}

/*
 * Prepare for output to a memory buffer.
 . The caller must have already allocated the buffer, and is responsible
 * for closing it after finishing compression.
 */
void jpeg_mem_dest (j_compress_ptr cinfo, char * outbuff, size_t * size)
{
  mem_dest_ptr dest;

  /* The destination object is made permanent so that multiple JPEG images
   * can be written to the same file without re-executing jpeg_stdio_dest.
   * This makes it dangerous to use this manager and a different destination
   * manager serially with the same JPEG object, because their private object
   * sizes may be different.  Caveat programmer.
   */
  if (cinfo->dest == NULL) {	/* first time for this JPEG object? */
    cinfo->dest = (struct jpeg_destination_mgr *)
      (*cinfo->mem->alloc_small) ((j_common_ptr) cinfo, JPOOL_PERMANENT,
				  sizeof(mem_destination_mgr));
  }

  dest = (mem_dest_ptr) cinfo->dest;
  dest->pub.init_destination = init_destination;
  dest->pub.empty_output_buffer = empty_output_buffer;
  dest->pub.term_destination = term_destination;
  dest->outbuff = (JOCTET *)outbuff;
  dest->size = (size_t *)size;
}

void ImgEncoder::writeJPEGFile(int quality, JSAMPLE * image_buffer, int image_width, int image_height)
{
  size_t buffersize = (image_width * image_height * 3) + JPEG_HEADER_PAD;
  char *jpeg_output_buffer = new char [buffersize];
  struct jpeg_compress_struct cinfo;
  struct jpeg_error_mgr jerr;
    
  JSAMPROW row_pointer[1];      // pointer to JSAMPLE row[s] 
  int row_stride;               // physical row width in image buffer 
    
  // Step 1: allocate and initialize JPEG compression object 
  cinfo.err = jpeg_std_error(&jerr);
  jpeg_create_compress(&cinfo);
    
  // Step 2: specify data destination (eg, a file) 
  //jpeg_stdio_dest(&cinfo, file->getFP());
  jpeg_mem_dest(&cinfo, jpeg_output_buffer, &buffersize);
    
  // Step 3: set parameters for compression 
  cinfo.image_width = image_width;      // image width and height, in pixels 
  cinfo.image_height = image_height;
  cinfo.input_components = 3;           // # of color components per pixel 
 
  cinfo.in_color_space = JCS_RGB;       // colorspace of input image 
    
  jpeg_set_defaults(&cinfo);
    
  jpeg_set_quality(&cinfo, quality, TRUE); // limit to baseline-JPEG values
    
  // Step 4: Start compressor 
  jpeg_start_compress(&cinfo, TRUE);
    
  // Step 5: while (scan lines remain to be written) 
  row_stride = image_width * 3; // JSAMPLEs per row in image_buffer 
    
  while (cinfo.next_scanline < cinfo.image_height) {
    row_pointer[0] = & image_buffer[cinfo.next_scanline * row_stride];
    (void) jpeg_write_scanlines(&cinfo, row_pointer, 1);
  }
    
  // Step 6: Finish compression 
  jpeg_finish_compress(&cinfo);

  // Step 7: release JPEG compression object 
  jpeg_destroy_compress(&cinfo);

  info->info("JPEG buffersize: %d", buffersize);
  file->Write(jpeg_output_buffer, buffersize);
  delete jpeg_output_buffer;
}

void ImgEncoder::getRGB(Frame *frame, unsigned char *rgb)
{
  unsigned char *pixels[3];
  int pitches[3];

  pixels[ 0 ] = rgb;
  pixels[ 1 ] = NULL;
  pixels[ 2 ] = NULL;

  pitches[ 0 ] = 720 * 3;
  pitches[ 1 ] = 0;
  pitches[ 2 ] = 0;
  
	dv_decoder_t *decoder = dv_decoder_new(FALSE/*this value is unused*/, FALSE, FALSE);
  decoder->quality = DV_QUALITY_BEST;

  dv_parse_header(decoder, frame->data);
  
  decoder->system = e_dv_system_625_50;  // PAL lines, PAL framerate
  decoder->sampling = e_dv_sample_422;  // 4 bytes y, 2 bytes u, 2 bytes v
  decoder->std = e_dv_std_iec_61834;
  decoder->num_dif_seqs = 12;
  
  // libdv img decode to rgb
  dv_decode_full_frame(decoder,
                       frame->data,
                       e_dv_color_rgb,
                       pixels,
                       pitches);
  
  dv_decoder_free(decoder);
}
