/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/***************************************************************************
 *            mov_encoder_thread.h
 *
 *  Tue May 17 16:00:01 CEST 2005
 *  Copyright  2005 Bent Bisballe
 *  deva@aasimon.org
 ****************************************************************************/

/*
 *    This file is part of MIaV.
 *
 *    MIaV is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MIaV is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MIaV; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 */

/*
 * $Id: mov_encoder_thread.h,v 1.8 2005/06/16 21:28:57 deva Exp $
 */

/*
 * $Log: mov_encoder_thread.h,v $
 * Revision 1.8  2005/06/16 21:28:57  deva
 * Rewrote thread object
 * Fixed bug in mov_encoder (pushed read_sem too many times, whihc lead to
 * growing server queue)
 *
 * Revision 1.7  2005/06/14 12:29:40  deva
 * Incorporated the use of the Info object everywhere... also using the log functionality.
 *
 * Revision 1.6  2005/06/09 11:00:03  deva
 * Added daemon code, and cleaned up using -Wall and -Werror
 *
 * Revision 1.5  2005/05/22 15:49:22  deva
 * Added multithreaded encoding support.
 *
 * Revision 1.4  2005/05/19 14:10:22  deva
 *
 * Multithreading rulez?
 *
 * Revision 1.3  2005/05/19 10:55:49  deva
 * Test for block encoding of length strlen("IPIPP").
 *
 * Revision 1.2  2005/05/17 15:12:51  deva
 * Fixed file rights (All read on files and directories, and all execute on directories).
 *
 * Revision 1.1  2005/05/17 14:30:56  deva
 * Added code, preparing threaded encoding.
 *
 */

#include <config.h>
#ifndef __MIAV_MOV_ENCODER_THREAD_H__
#define __MIAV_MOV_ENCODER_THREAD_H__

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <vector>
using namespace std;

#include "mov_encoder.h"
#include "mov_encoder_writer.h"

#include "info.h"

class MovEncoderThread {
public:
  MovEncoderThread(const char *cpr, Info *info);
  ~MovEncoderThread();

  void encode(Frame* frame);

private:
  Info *info;

  FrameVectorQueue *inputqueue;
  FramePriorityQueue *outputqueue;
  FrameVector *block;

  //thread stuff
  sem_t in_sem;
  sem_t out_sem;

  sem_t read_sem;

  pthread_mutex_t input_mutex;
  pthread_mutex_t output_mutex;

  // Used for encoder switching
  unsigned int frame_number;

  unsigned int num_frames_in_block;

  MovEncoderWriter *writer;
  //  pthread_t* writer_tid;

  int threads;
  vector<MovEncoder*> encs;
  //  vector<pthread_t*> tids;
};

#endif/*__MIAV_MOV_ENCODER_THREAD_H__*/
