/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/***************************************************************************
 *            mov_encoder_writer.cc
 *
 *  Sun May 22 12:51:36 CEST 2005
 *  Copyright  2005 Bent Bisballe
 *  deva@aasimon.org
 ****************************************************************************/

/*
 *    This file is part of MIaV.
 *
 *    MIaV is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MIaV is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MIaV; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 */

/*
 * $Id: mov_encoder_writer.cc,v 1.7 2005/06/19 20:04:43 deva Exp $
 */

/*
 * $Log: mov_encoder_writer.cc,v $
 * Revision 1.7  2005/06/19 20:04:43  deva
 * ImgEncoder now uses the file class for output, through jpeg_mem_dest.
 *
 * Revision 1.6  2005/06/16 21:28:57  deva
 * Rewrote thread object
 * Fixed bug in mov_encoder (pushed read_sem too many times, whihc lead to
 * growing server queue)
 *
 * Revision 1.5  2005/06/14 18:58:35  deva
 * *** empty log message ***
 *
 * Revision 1.4  2005/06/14 12:29:40  deva
 * Incorporated the use of the Info object everywhere... also using the log functionality.
 *
 * Revision 1.3  2005/05/26 21:32:39  deva
 * *** empty log message ***
 *
 * Revision 1.2  2005/05/26 12:48:36  deva
 * *** empty log message ***
 *
 * Revision 1.1  2005/05/22 15:49:22  deva
 * Added multithreaded encoding support.
 *
 */
#include <config.h>
#include "mov_encoder_writer.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <pthread.h>
#include <semaphore.h>

#include <errno.h>

#include <string>
using namespace std;

#include "miav_config.h"

#include <time.h>

MovEncoderWriter::MovEncoderWriter(const char* cpr, 
                                   FramePriorityQueue *q, 
                                   sem_t *s,
                                   pthread_mutex_t *m, 
                                   Info *i)
{
  info = i;
  info->info("MovEncoderWriter");

  // Create path and filename
  char fname[256];
  string *server_root;
  char birthmonth[3];
  char date[32];

  // Get server root
  server_root = config->readString("server_root");

  // Copy the bytes representing the birth month from the cpr
  // [dd][mm][yy]-[nn][nn]
  strncpy(birthmonth, &cpr[2], 2);
  birthmonth[2] = 0;

  // Create date (today) in [yyyy][mm][dd]
  struct tm *ltime;
  time_t t = time(NULL);
  ltime = localtime(&t);
  sprintf(date, "%.4d%.2d%.2d", 
          ltime->tm_year + 1900, 
          ltime->tm_mon, 
          ltime->tm_mday);

  sprintf(fname, "%s/%s/%s/%s-%s-", server_root->c_str(), birthmonth, cpr, cpr, date);

  file = new File(fname, "mpg", info);

  sem = s;
  queue = q;
  frame_number = 0;
  mutex = m;

  running = true;
}

MovEncoderWriter::~MovEncoderWriter()
{
  info->info("~MovEncoderWriter");
  delete file;
}

void MovEncoderWriter::thread_main()
{
  info->info("MovEncoderWriter::run");

  Frame *frame;

  while(running) {
    sem_wait(sem);

    //    int poolsize;

    // Lock output mutex
    pthread_mutex_lock(mutex);
    frame = queue->top();
    if(frame && frame->number == frame_number) queue->pop();
    //    poolsize = queue->size();
    pthread_mutex_unlock(mutex);
    // Unlock output mutex

    int wrote = 0;
    while(frame && (frame->number == frame_number)) {
      
      int ret;

      ret = file->Write(frame->data, frame->size);
      frame_number++;
      wrote ++;

      delete frame;

      if(ret == -1) {
        info->error("File write returned -1.");
        return;
      }
      
      // Lock output mutex
      pthread_mutex_lock(mutex);
      frame = queue->top();
      if(frame && frame->number == frame_number) queue->pop();
      //      poolsize = queue->size();
      pthread_mutex_unlock(mutex);
      // Unlock output mutex
    }

    //    info->info("Wrote %d frames, pool size %d - exp: %d", wrote, poolsize, frame_number);
  }

  info->info("MovEncoderWriter::stop");
}
