/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/***************************************************************************
 *            server_status.cc
 *
 *  Fri Apr 29 13:58:26 CEST 2005
 *  Copyright  2005 Bent Bisballe
 *  deva@aasimon.org
 ****************************************************************************/

/*
 *    This file is part of MIaV.
 *
 *    MIaV is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MIaV is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MIaV; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 */

/*
 * $Id: server_status.cc,v 1.10 2005/06/19 20:04:43 deva Exp $
 */

/*
 * $Log: server_status.cc,v $
 * Revision 1.10  2005/06/19 20:04:43  deva
 * ImgEncoder now uses the file class for output, through jpeg_mem_dest.
 *
 * Revision 1.9  2005/06/19 11:44:14  deva
 * Cleaned up a log of logging.
 * Fixed server queue (shouldn't happen).
 * Added user and group lookup.
 *
 * Revision 1.8  2005/05/22 15:49:22  deva
 * Added multithreaded encoding support.
 *
 * Revision 1.7  2005/05/17 15:12:51  deva
 * Fixed file rights (All read on files and directories, and all execute on directories).
 *
 * Revision 1.6  2005/05/16 16:00:57  deva
 * Lots of stuff!
 *
 * Revision 1.5  2005/05/16 13:25:52  deva
 * Moved video setting to configuration file.
 * Fine tuned setting for 2.4ghz server
 *
 * Revision 1.4  2005/05/09 16:40:20  deva
 * Added optimize yuv conversion code
 *
 * Revision 1.3  2005/05/03 08:31:59  deva
 * Removed the error object, and replaced it with a more generic info object.
 *
 * Revision 1.2  2005/05/01 09:56:26  deva
 * Added Id and Log tags to all files
 */

#include <config.h>
#include "server_status.h"

#include <stdio.h>

ServerStatus::ServerStatus(Info *i)
{
  info = i;

  gettimeofday(&oldtime, NULL);
  
  for(int cnt = 0; cnt < BUFFERSIZE; cnt++) {
    frametime[cnt] = 41660;
  }

  gettimeofday(&time, NULL);

  interval = 0;
}

ServerStatus::~ServerStatus()
{
}

void ServerStatus::checkPoint()
{
  for(int cnt = BUFFERSIZE - 1; cnt > 0; cnt--) {
    frametime[cnt] = frametime[cnt-1];
  }
  frametime[0] = (1000000 * time.tv_sec + time.tv_usec) - (1000000 * oldtime.tv_sec + oldtime.tv_usec);

  oldtime.tv_sec = time.tv_sec;
  oldtime.tv_usec = time.tv_usec;

  gettimeofday(&time, NULL);

  interval += frametime[0];
  if(interval > UPD) {
    interval = 0;
    double total = 0.0;
    for(int cnt = 0; cnt < BUFFERSIZE; cnt++) {
      total += (double)frametime[cnt];
    }
    info->info("Status - fps: %f", 1000000.0 / (total / (double)BUFFERSIZE));
  }
  
}

/*
date(1), gettimeofday(2), ctime(3), ftime(3)
*/
